/*
 * @Descripttion:实名认证
 * @version:
 * @Author: Neo,Huang
 * @Date: 2021-04-17 11:04:16
 * @LastEditors: Neo,Huang
 * @LastEditTime: 2021-05-06 14:46:17
 */
package nppa

import (
	"fmt"
	"log"
	"time"

	"github.com/bitly/go-simplejson"
)

// 实名认证 - 测试校验
func TestCheckPlayerId(testCode string, uid string, name string, idNum string) map[string]interface{} {
	ret := make(map[string]interface{})
	ret["code"] = 400

	url := fmt.Sprintf("https://wlc.nppa.gov.cn/test/authentication/check/%s", testCode)
	log.Println("TestCheckPlayerId url:", url)

	chn := -1
	mapHead := ConfGetInfoByChannel(chn)
	if mapHead == nil {
		// 未响应
		ret["msg"] = "配置错误"
		return ret
	}

	mapBody := make(map[string]interface{})
	mapBody["ai"] = uid
	mapBody["name"] = name
	mapBody["idNum"] = idNum

	body := MakeRequestBody(mapBody, mapHead["secretKey"])
	sign := MakeRequestSign(mapHead, nil, body)
	log.Println(fmt.Sprintf("TestCheckPlayerId body[%s] sign[%s]", body, sign))
	mapHead["sign"] = sign

	buff, _ := Send2Nppa("POST", url, mapHead, nil, body)
	j, err := simplejson.NewJson(buff)
	if err != nil {
		log.Println("TestCheckPlayerId err", err)
		// 未响应
		ret["msg"] = "未响应"
		return ret
	}
	code, _ := j.Get("errcode").Int()
	if code == 0 {
		ret["code"] = 200
	} else {
		log.Println("TestCheckPlayerId ret:", j)
	}
	ret["msg"], _ = j.Get("errmsg").String()

	return ret
}

// 实名认证 - 测试查询结果
func TestQueryPlayerId(testCode string, uid string) map[string]interface{} {
	ret := make(map[string]interface{})
	ret["code"] = 400

	url := fmt.Sprintf("https://wlc.nppa.gov.cn/test/authentication/query/%s", testCode)
	log.Println("TestQueryPlayerId url:", url)

	chn := -1
	mapHead := ConfGetInfoByChannel(chn)
	if mapHead == nil {
		// 未响应
		ret["msg"] = "配置错误"
		return ret
	}

	mapParam := make(map[string]string)
	mapParam["ai"] = uid

	sign := MakeRequestSign(mapHead, mapParam, "")
	log.Println(fmt.Sprintf("TestQueryPlayerId uid[%s] sign[%s]", uid, sign))
	mapHead["sign"] = sign

	buff, _ := Send2Nppa("GET", url, mapHead, mapParam, "")
	j, err := simplejson.NewJson(buff)
	if err != nil {
		log.Println("TestQueryPlayerId err", err)
		// 未响应
		ret["msg"] = "未响应"

		return ret
	}
	code, _ := j.Get("errcode").Int()
	if code == 0 {
		ret["code"] = 200
	} else {
		log.Println("TestQueryPlayerId ret:", j)
	}
	ret["msg"], _ = j.Get("errmsg").String()

	return ret
}

// 实名认证 - 测试数据上报
func TestLoginout(testCode string, si string, bt int, ct int, di string, pi string) map[string]interface{} {
	ret := make(map[string]interface{})
	ret["code"] = 400

	url := fmt.Sprintf("https://wlc.nppa.gov.cn/test/collection/loginout/%s", testCode)
	log.Println("TestLoginout url:", url)

	chn := -1
	mapHead := ConfGetInfoByChannel(chn)
	if mapHead == nil {
		// 未响应
		ret["msg"] = "配置错误"
		return ret
	}

	mapBody := make(map[string]interface{})
	collections := []map[string]interface{}{}
	collection := make(map[string]interface{})
	collection["no"] = 1
	collection["si"] = si
	collection["bt"] = bt
	collection["ot"] = time.Now().Unix()
	collection["ct"] = ct
	if ct == 0 {
		collection["pi"] = pi
	} else if ct == 2 {
		collection["di"] = di
	}
	collections = append(collections, collection)
	mapBody["collections"] = collections

	body := MakeRequestBody(mapBody, mapHead["secretKey"])
	sign := MakeRequestSign(mapHead, nil, body)
	log.Println(fmt.Sprintf("TestLoginout body[%s] sign[%s]", body, sign))
	mapHead["sign"] = sign

	buff, _ := Send2Nppa("POST", url, mapHead, nil, body)
	j, err := simplejson.NewJson(buff)
	if err != nil {
		log.Println("TestLoginout err", err)
		// 未响应
		ret["msg"] = "未响应"

		return ret
	}
	code, _ := j.Get("errcode").Int()
	if code == 0 {
		ret["code"] = 200
	} else {
		log.Println("TestLoginout ret:", j)
	}
	ret["msg"], _ = j.Get("errmsg").String()

	return ret
}
